/*
 * This file is part of the libCEC(R) library.
 *
 * libCEC(R) is Copyright (C) 2011-2013 Pulse-Eight Limited.  All rights reserved.
 * libCEC(R) is an original work, containing original code.
 *
 * libCEC(R) is a trademark of Pulse-Eight Limited.
 *
 * This program is dual-licensed; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *
 * Alternatively, you can license this library under a commercial license,
 * please contact Pulse-Eight Licensing for more information.
 *
 * For more information contact:
 * Pulse-Eight Licensing       <license@pulse-eight.com>
 *     http://www.pulse-eight.com/
 *     http://www.pulse-eight.net/
 */

using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using LibCECTray.ui;

namespace LibCECTray.controller.applications
{
  partial class CecButtonConfigUI : Form
  {
    public CecButtonConfigUI(CecButtonConfigItem button)
    {
      _button = button;
      InitializeComponent();

      var keys = Enum.GetValues(typeof (WindowsAPI.VirtualKeyCode));
      string[] friendlyNames = new string[keys.Length];
      for (int i = 0; i < keys.Length; i++)
        friendlyNames[i] = WindowsAPI.GetVirtualKeyName((WindowsAPI.VirtualKeyCode)keys.GetValue(i));
      Array.Sort(friendlyNames);
      foreach (var item in friendlyNames)
        cbAddKey.Items.Add(item);

      button.SettingChanged += delegate
                                 {
                                   _lastSelectedText = 0;
                                   tbAction.Text = button.Value.AsFriendlyString().Trim();
                                 };

      lButtonName.Text = button.Label.Text;
      tbAction.Text = button.Value.AsFriendlyString().Trim();

      //TODO
      cbAddAction.Items.Add(ApplicationInput.FriendlyActionName(ActionType.CloseControllerApplication));
      cbAddAction.Items.Add(ApplicationInput.FriendlyActionName(ActionType.StartApplication));

      // take the icon of the main window
      ComponentResourceManager resources = new ComponentResourceManager(typeof(CECTray));
      Icon = resources.GetObject("$this.Icon") as Icon;
    }

    private void ButtonControlCheckSelection(object sender, MouseEventArgs e)
    {
      ButtonControlCheckSelection();
    }

    private void ButtonControlCheckSelection(object sender, KeyEventArgs e)
    {
      ButtonControlCheckSelection();
    }

    private void ButtonControlCheckSelection()
    {
      if (tbAction.SelectionStart != _lastSelectedText)
      {
        _lastSelectedText = SelectionChanged(tbAction.SelectionStart) ? 0 : tbAction.SelectionStart;
      }
    }

    private bool SelectionChanged(int selection)
    {
      _button.Value = _button.Value.RemoveItem(selection);
      return true;
    }

    public CecButtonConfigUI()
    {
      InitializeComponent();
    }

    private void BCloseClick(object sender, EventArgs e)
    {
      Hide();
    }

    private void BDefaultClick(object sender, EventArgs e)
    {
      _button.Value = _button.DefaultValue;
    }

    private void BClearClick(object sender, EventArgs e)
    {
      _button.Value = null;
    }

    private void BAddKeyClick(object sender, EventArgs e)
    {
      foreach (var item in Enum.GetValues(typeof(WindowsAPI.VirtualKeyCode)))
      {
        if (cbAddKey.Text.Equals(WindowsAPI.GetVirtualKeyName((WindowsAPI.VirtualKeyCode)item)))
        {
          _button.Value = _button.Value.AddKey((WindowsAPI.VirtualKeyCode) item);
          break;
        }
      }
    }

    private void BAddActionClick(object sender, EventArgs e)
    {
      foreach (var item in Enum.GetValues(typeof(ActionType)))
      {
        if (cbAddAction.Text.ToUpper().Equals(ApplicationInput.FriendlyActionName((ActionType)item).ToUpper()))
        {
          _button.Value = _button.Value.AddAction((ActionType)item);
          break;
        }
      }
    }

    private int _lastSelectedText;
    private readonly CecButtonConfigItem _button;
  }
}
